@@ -4,16 +4,19 @@
 //! user configuration file.
 
 use anyhow::{anyhow, Context, Result};
+use dialoguer::Confirm;
 use dirs::{config_dir, home_dir};
 use log::{debug, trace};
-use pimalaya_email::{AccountConfig, BackendConfig, EmailHooks, EmailTextPlainFormat};
+use pimalaya_email::{AccountConfig, EmailHooks, EmailTextPlainFormat};
+use pimalaya_process::Cmd;
 use serde::{Deserialize, Serialize};
-use std::{collections::HashMap, fs, path::PathBuf};
+use std::{collections::HashMap, fs, path::PathBuf, process};
 use toml;
 
 use crate::{
     account::DeserializedAccountConfig,
-    config::{prelude::*, wizard::wizard},
+    config::{prelude::*, wizard},
+    wizard_prompt, wizard_warn,
 };
 
 /// Represents the user config file.
@@ -31,13 +34,38 @@ pub struct DeserializedConfig {
 
     pub email_listing_page_size: Option<usize>,
     pub email_reading_headers: Option<Vec<String>>,
-    #[serde(default, with = "EmailTextPlainFormatDef")]
+    #[serde(
+        default,
+        with = "EmailTextPlainFormatDef",
+        skip_serializing_if = "EmailTextPlainFormat::is_default"
+    )]
     pub email_reading_format: EmailTextPlainFormat,
-    pub email_reading_verify_cmd: Option<String>,
-    pub email_reading_decrypt_cmd: Option<String>,
+    #[serde(
+        default,
+        with = "OptionCmdDef",
+        skip_serializing_if = "Option::is_none"
+    )]
+    pub email_reading_verify_cmd: Option<Cmd>,
+    #[serde(
+        default,
+        with = "OptionCmdDef",
+        skip_serializing_if = "Option::is_none"
+    )]
+    pub email_reading_decrypt_cmd: Option<Cmd>,
     pub email_writing_headers: Option<Vec<String>>,
-    pub email_writing_sign_cmd: Option<String>,
-    pub email_writing_encrypt_cmd: Option<String>,
+    #[serde(
+        default,
+        with = "OptionCmdDef",
+        skip_serializing_if = "Option::is_none"
+    )]
+    pub email_writing_sign_cmd: Option<Cmd>,
+    #[serde(
+        default,
+        with = "OptionCmdDef",
+        skip_serializing_if = "Option::is_none"
+    )]
+    pub email_writing_encrypt_cmd: Option<Cmd>,
+    pub email_sending_save_copy: Option<bool>,
     #[serde(
         default,
         with = "EmailHooksDef",
@@ -54,12 +82,24 @@ impl DeserializedConfig {
     pub fn from_opt_path(path: Option<&str>) -> Result<Self> {
         debug!("path: {:?}", path);
 
-        let config: Self = match path.map(|s| s.into()).or_else(Self::path) {
-            Some(path) => {
-                let content = fs::read_to_string(path).context("cannot read config file")?;
-                toml::from_str(&content).context("cannot parse config file")?
+        let config = if let Some(path) = path.map(PathBuf::from).or_else(Self::path) {
+            let content = fs::read_to_string(path).context("cannot read config file")?;
+            toml::from_str(&content).context("cannot parse config file")?
+        } else {
+            wizard_warn!("Himalaya could not find an already existing configuration file.");
+
+            if !Confirm::new()
+                .with_prompt(wizard_prompt!(
+                    "Would you like to create one with the wizard?"
+                ))
+                .default(true)
+                .interact_opt()?
+                .unwrap_or_default()
+            {
+                process::exit(0);
             }
-            None => wizard()?,
+
+            wizard::configure()?
         };
 
         if config.accounts.is_empty() {
@@ -90,17 +130,16 @@ impl DeserializedConfig {
             .filter(|p| p.exists())
     }
 
-    pub fn to_configs(&self, account_name: Option<&str>) -> Result<(AccountConfig, BackendConfig)> {
+    pub fn to_account_config(&self, account_name: Option<&str>) -> Result<AccountConfig> {
         let (account_name, deserialized_account_config) = match account_name {
             Some("default") | Some("") | None => self
                 .accounts
                 .iter()
                 .find_map(|(name, account)| {
-                    if account.is_default() {
-                        Some((name.clone(), account))
-                    } else {
-                        None
-                    }
+                    account
+                        .default
+                        .filter(|default| *default == true)
+                        .map(|_| (name.clone(), account))
                 })
                 .ok_or_else(|| anyhow!("cannot find default account")),
             Some(name) => self
@@ -110,33 +149,27 @@ impl DeserializedConfig {
                 .ok_or_else(|| anyhow!(format!("cannot find account {}", name))),
         }?;
 
-        let (account_config, backend_config) =
-            deserialized_account_config.to_configs(account_name, self);
-
-        Ok((account_config, backend_config))
+        Ok(deserialized_account_config.to_account_config(account_name, self))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use pimalaya_email::{EmailSender, MaildirConfig, SendmailConfig, SmtpConfig};
+    use pimalaya_email::{
+        BackendConfig, MaildirConfig, PasswdConfig, SenderConfig, SendmailConfig,
+    };
+    use pimalaya_secret::Secret;
 
-    #[cfg(feature = "imap-backend")]
-    use pimalaya_email::ImapConfig;
     #[cfg(feature = "notmuch-backend")]
     use pimalaya_email::NotmuchConfig;
+    #[cfg(feature = "imap-backend")]
+    use pimalaya_email::{ImapAuthConfig, ImapConfig};
+    #[cfg(feature = "smtp-sender")]
+    use pimalaya_email::{SmtpAuthConfig, SmtpConfig};
 
     use std::io::Write;
     use tempfile::NamedTempFile;
 
-    use crate::account::DeserializedBaseAccountConfig;
-
-    #[cfg(feature = "imap-backend")]
-    use crate::account::DeserializedImapAccountConfig;
-    use crate::account::DeserializedMaildirAccountConfig;
-    #[cfg(feature = "notmuch-backend")]
-    use crate::account::DeserializedNotmuchAccountConfig;
-
     use super::*;
 
     fn make_config(config: &str) -> Result<DeserializedConfig> {
@@ -156,9 +189,23 @@ mod tests {
     }
 
     #[test]
-    fn account_missing_backend_field() {
+    fn account_missing_email_field() {
         let config = make_config("[account]");
 
+        assert!(config
+            .unwrap_err()
+            .root_cause()
+            .to_string()
+            .contains("missing field `email`"));
+    }
+
+    #[test]
+    fn account_missing_backend_field() {
+        let config = make_config(
+            "[account]
+            email = \"test@localhost\"",
+        );
+
         assert!(config
             .unwrap_err()
             .root_cause()
@@ -170,6 +217,7 @@ mod tests {
     fn account_invalid_backend_field() {
         let config = make_config(
             "[account]
+            email = \"test@localhost\"
             backend = \"bad\"",
         );
 
@@ -180,20 +228,6 @@ mod tests {
             .contains("unknown variant `bad`"));
     }
 
-    #[test]
-    fn account_missing_email_field() {
-        let config = make_config(
-            "[account]
-            backend = \"none\"",
-        );
-
-        assert!(config
-            .unwrap_err()
-            .root_cause()
-            .to_string()
-            .contains("missing field `email`"));
-    }
-
     #[test]
     fn imap_account_missing_host_field() {
         let config = make_config(
@@ -261,7 +295,7 @@ mod tests {
             .unwrap_err()
             .root_cause()
             .to_string()
-            .contains("missing field `imap-passwd-cmd`"));
+            .contains("missing field `imap-auth`"));
     }
 
     #[test]
@@ -380,7 +414,7 @@ mod tests {
     }
 
     #[test]
-    fn account_smtp_sender_missing_passwd_cmd_field() {
+    fn account_smtp_sender_missing_auth_field() {
         let config = make_config(
             "[account]
             email = \"test@localhost\"
@@ -395,7 +429,7 @@ mod tests {
             .unwrap_err()
             .root_cause()
             .to_string()
-            .contains("missing field `smtp-passwd-cmd`"));
+            .contains("missing field `smtp-auth`"));
     }
 
     #[test]
@@ -414,8 +448,11 @@ mod tests {
             .contains("missing field `sendmail-cmd`"));
     }
 
+    #[cfg(feature = "smtp-sender")]
     #[test]
     fn account_smtp_sender_minimum_config() {
+        use pimalaya_email::SenderConfig;
+
         let config = make_config(
             "[account]
             email = \"test@localhost\"
@@ -424,7 +461,8 @@ mod tests {
             smtp-host = \"localhost\"
             smtp-port = 25
             smtp-login = \"login\"
-            smtp-passwd-cmd = \"echo password\"",
+            smtp-auth = \"passwd\"
+            smtp-passwd = { cmd  = \"echo password\" }",
         );
 
         assert_eq!(
@@ -432,17 +470,19 @@ mod tests {
             DeserializedConfig {
                 accounts: HashMap::from_iter([(
                     "account".into(),
-                    DeserializedAccountConfig::None(DeserializedBaseAccountConfig {
+                    DeserializedAccountConfig {
                         email: "test@localhost".into(),
-                        email_sender: EmailSender::Smtp(SmtpConfig {
+                        sender: SenderConfig::Smtp(SmtpConfig {
                             host: "localhost".into(),
                             port: 25,
                             login: "login".into(),
-                            passwd_cmd: "echo password".into(),
+                            auth: SmtpAuthConfig::Passwd(PasswdConfig {
+                                passwd: Secret::new_cmd(String::from("echo password"))
+                            }),
                             ..SmtpConfig::default()
                         }),
-                        ..DeserializedBaseAccountConfig::default()
-                    })
+                        ..DeserializedAccountConfig::default()
+                    }
                 )]),
                 ..DeserializedConfig::default()
             }
@@ -464,13 +504,13 @@ mod tests {
             DeserializedConfig {
                 accounts: HashMap::from_iter([(
                     "account".into(),
-                    DeserializedAccountConfig::None(DeserializedBaseAccountConfig {
+                    DeserializedAccountConfig {
                         email: "test@localhost".into(),
-                        email_sender: EmailSender::Sendmail(SendmailConfig {
-                            cmd: "echo send".into(),
+                        sender: SenderConfig::Sendmail(SendmailConfig {
+                            cmd: Cmd::from("echo send")
                         }),
-                        ..DeserializedBaseAccountConfig::default()
-                    })
+                        ..DeserializedAccountConfig::default()
+                    }
                 )]),
                 ..DeserializedConfig::default()
             }
@@ -487,7 +527,8 @@ mod tests {
             imap-host = \"localhost\"
             imap-port = 993
             imap-login = \"login\"
-            imap-passwd-cmd = \"echo password\"",
+            imap-auth = \"passwd\"
+            imap-passwd = { cmd = \"echo password\" }",
         );
 
         assert_eq!(
@@ -495,19 +536,19 @@ mod tests {
             DeserializedConfig {
                 accounts: HashMap::from_iter([(
                     "account".into(),
-                    DeserializedAccountConfig::Imap(DeserializedImapAccountConfig {
-                        base: DeserializedBaseAccountConfig {
-                            email: "test@localhost".into(),
-                            ..DeserializedBaseAccountConfig::default()
-                        },
-                        backend: ImapConfig {
+                    DeserializedAccountConfig {
+                        email: "test@localhost".into(),
+                        backend: BackendConfig::Imap(ImapConfig {
                             host: "localhost".into(),
                             port: 993,
                             login: "login".into(),
-                            passwd_cmd: "echo password".into(),
+                            auth: ImapAuthConfig::Passwd(PasswdConfig {
+                                passwd: Secret::new_cmd(String::from("echo password"))
+                            }),
                             ..ImapConfig::default()
-                        }
-                    })
+                        }),
+                        ..DeserializedAccountConfig::default()
+                    }
                 )]),
                 ..DeserializedConfig::default()
             }
@@ -528,15 +569,13 @@ mod tests {
             DeserializedConfig {
                 accounts: HashMap::from_iter([(
                     "account".into(),
-                    DeserializedAccountConfig::Maildir(DeserializedMaildirAccountConfig {
-                        base: DeserializedBaseAccountConfig {
-                            email: "test@localhost".into(),
-                            ..DeserializedBaseAccountConfig::default()
-                        },
-                        backend: MaildirConfig {
+                    DeserializedAccountConfig {
+                        email: "test@localhost".into(),
+                        backend: BackendConfig::Maildir(MaildirConfig {
                             root_dir: "/tmp/maildir".into(),
-                        }
-                    })
+                        }),
+                        ..DeserializedAccountConfig::default()
+                    }
                 )]),
                 ..DeserializedConfig::default()
             }
@@ -559,15 +598,13 @@ mod tests {
             DeserializedConfig {
                 accounts: HashMap::from_iter([(
                     "account".into(),
-                    DeserializedAccountConfig::Notmuch(DeserializedNotmuchAccountConfig {
-                        base: DeserializedBaseAccountConfig {
-                            email: "test@localhost".into(),
-                            ..DeserializedBaseAccountConfig::default()
-                        },
-                        backend: NotmuchConfig {
+                    DeserializedAccountConfig {
+                        email: "test@localhost".into(),
+                        backend: BackendConfig::Notmuch(NotmuchConfig {
                             db_path: "/tmp/notmuch.db".into(),
-                        }
-                    })
+                        }),
+                        ..DeserializedAccountConfig::default()
+                    }
                 )]),
                 ..DeserializedConfig::default()
             }
