@@ -4,18 +4,79 @@
 
 use anyhow::Result;
 use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
-use log::{info, trace};
+use log::{info, trace, warn};
 use pimalaya_email::{
-    folder::sync::Strategy as SyncFoldersStrategy, AccountConfig, Backend, BackendSyncBuilder,
-    BackendSyncProgressEvent,
+    folder::sync::Strategy as SyncFoldersStrategy, AccountConfig, Backend, BackendConfig,
+    BackendSyncBuilder, BackendSyncProgressEvent, ImapAuthConfig, SenderConfig, SmtpAuthConfig,
 };
 
 use crate::{
-    config::DeserializedConfig,
+    config::{
+        wizard::{prompt_passwd, prompt_secret},
+        DeserializedConfig,
+    },
     printer::{PrintTableOpts, Printer},
     Accounts,
 };
 
+/// Configure the current selected account
+pub fn configure(config: &AccountConfig, reset: bool) -> Result<()> {
+    info!("entering the configure account handler");
+
+    if reset {
+        #[cfg(feature = "imap-backend")]
+        if let BackendConfig::Imap(imap_config) = &config.backend {
+            let reset = match &imap_config.auth {
+                ImapAuthConfig::Passwd(passwd) => passwd.reset(),
+                ImapAuthConfig::OAuth2(oauth2) => oauth2.reset(),
+            };
+            if let Err(err) = reset {
+                warn!("error while resetting imap secrets, skipping it");
+                warn!("{err}");
+            }
+        }
+
+        #[cfg(feature = "smtp-sender")]
+        if let SenderConfig::Smtp(smtp_config) = &config.sender {
+            let reset = match &smtp_config.auth {
+                SmtpAuthConfig::Passwd(passwd) => passwd.reset(),
+                SmtpAuthConfig::OAuth2(oauth2) => oauth2.reset(),
+            };
+            if let Err(err) = reset {
+                warn!("error while resetting smtp secrets, skipping it");
+                warn!("{err}");
+            }
+        }
+    }
+
+    #[cfg(feature = "imap-backend")]
+    if let BackendConfig::Imap(imap_config) = &config.backend {
+        match &imap_config.auth {
+            ImapAuthConfig::Passwd(passwd) => passwd.configure(|| prompt_passwd("IMAP password")),
+            ImapAuthConfig::OAuth2(oauth2) => {
+                oauth2.configure(|| prompt_secret("IMAP OAuth 2.0 client secret"))
+            }
+        }?;
+    }
+
+    #[cfg(feature = "smtp-sender")]
+    if let SenderConfig::Smtp(smtp_config) = &config.sender {
+        match &smtp_config.auth {
+            SmtpAuthConfig::Passwd(passwd) => passwd.configure(|| prompt_passwd("SMTP password")),
+            SmtpAuthConfig::OAuth2(oauth2) => {
+                oauth2.configure(|| prompt_secret("SMTP OAuth 2.0 client secret"))
+            }
+        }?;
+    }
+
+    println!(
+        "Account successfully {}configured!",
+        if reset { "re" } else { "" }
+    );
+
+    Ok(())
+}
+
 /// Lists all accounts.
 pub fn list<'a, P: Printer>(
     max_width: Option<usize>,
@@ -231,9 +292,7 @@ mod tests {
     use termcolor::ColorSpec;
 
     use crate::{
-        account::{
-            DeserializedAccountConfig, DeserializedBaseAccountConfig, DeserializedImapAccountConfig,
-        },
+        account::DeserializedAccountConfig,
         printer::{Print, PrintTable, WriteColor},
     };
 
@@ -305,13 +364,11 @@ mod tests {
         let deserialized_config = DeserializedConfig {
             accounts: HashMap::from_iter([(
                 "account-1".into(),
-                DeserializedAccountConfig::Imap(DeserializedImapAccountConfig {
-                    base: DeserializedBaseAccountConfig {
-                        default: Some(true),
-                        ..DeserializedBaseAccountConfig::default()
-                    },
-                    backend: ImapConfig::default(),
-                }),
+                DeserializedAccountConfig {
+                    default: Some(true),
+                    backend: BackendConfig::Imap(ImapConfig::default()),
+                    ..DeserializedAccountConfig::default()
+                },
             )]),
             ..DeserializedConfig::default()
         };
