@@ -2,7 +2,7 @@ use anyhow::{anyhow, Context, Result};
 use atty::Stream;
 use log::{debug, trace};
 use pimalaya_email::{
-    AccountConfig, Backend, Email, Flag, Flags, Sender, ShowTextPartsStrategy, Tpl, TplBuilder,
+    AccountConfig, Backend, Email, EmailBuilder, FilterParts, Flag, Flags, Sender,
 };
 use std::{
     fs,
@@ -14,17 +14,20 @@ use uuid::Uuid;
 use crate::{
     printer::{PrintTableOpts, Printer},
     ui::editor,
-    Envelopes,
+    Envelopes, IdMapper,
 };
 
-pub fn attachments<P: Printer, B: Backend + ?Sized>(
+pub fn attachments<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     ids: Vec<&str>,
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
+    let ids = id_mapper.get_ids(ids)?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
     let emails = backend.get_emails(&folder, ids.clone())?;
     let mut index = 0;
 
@@ -71,60 +74,72 @@ pub fn attachments<P: Printer, B: Backend + ?Sized>(
     }
 }
 
-pub fn copy<P: Printer, B: Backend + ?Sized>(
+pub fn copy<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     from_folder: &str,
     to_folder: &str,
     ids: Vec<&str>,
 ) -> Result<()> {
     let from_folder = config.folder_alias(from_folder)?;
     let to_folder = config.folder_alias(to_folder)?;
+    let ids = id_mapper.get_ids(ids)?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
     backend.copy_emails(&from_folder, &to_folder, ids)?;
     printer.print("Email(s) successfully copied!")
 }
 
-pub fn delete<P: Printer, B: Backend + ?Sized>(
+pub fn delete<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     ids: Vec<&str>,
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
+    let ids = id_mapper.get_ids(ids)?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
     backend.delete_emails(&folder, ids)?;
     printer.print("Email(s) successfully deleted!")
 }
 
-pub fn forward<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn forward<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
     folder: &str,
     id: &str,
-    headers: Option<Vec<&str>>,
+    headers: Option<Vec<(&str, &str)>>,
     body: Option<&str>,
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
+
+    let ids = id_mapper.get_ids([id])?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
+
     let tpl = backend
-        .get_emails(&folder, vec![id])?
+        .get_emails(&folder, ids)?
         .first()
         .ok_or_else(|| anyhow!("cannot find email {}", id))?
-        .to_forward_tpl_builder(config)?
-        .set_some_raw_headers(headers)
-        .some_text_plain_part(body)
-        .build();
+        .to_forward_tpl_builder(config)
+        .some_headers(headers)
+        .some_body(body)
+        .build()?;
     trace!("initial template: {}", *tpl);
     editor::edit_tpl_with_editor(config, printer, backend, sender, tpl)?;
     Ok(())
 }
 
-pub fn list<P: Printer, B: Backend + ?Sized>(
+pub fn list<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     max_width: Option<usize>,
     page_size: Option<usize>,
@@ -133,8 +148,11 @@ pub fn list<P: Printer, B: Backend + ?Sized>(
     let folder = config.folder_alias(folder)?;
     let page_size = page_size.unwrap_or(config.email_listing_page_size());
     debug!("page size: {}", page_size);
-    let envelopes: Envelopes = backend.list_envelopes(&folder, page_size, page)?.into();
+
+    let mut envelopes: Envelopes = backend.list_envelopes(&folder, page_size, page)?.into();
+    envelopes.remap_ids(id_mapper)?;
     trace!("envelopes: {:?}", envelopes);
+
     printer.print_table(
         Box::new(envelopes),
         PrintTableOpts {
@@ -147,54 +165,64 @@ pub fn list<P: Printer, B: Backend + ?Sized>(
 /// Parses and edits a message from a [mailto] URL string.
 ///
 /// [mailto]: https://en.wikipedia.org/wiki/Mailto
-pub fn mailto<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn mailto<P: Printer>(
     config: &AccountConfig,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
     url: &Url,
 ) -> Result<()> {
-    let mut tpl = TplBuilder::default().to(url.path());
+    let mut builder = EmailBuilder::new().to(url.path());
 
     for (key, val) in url.query_pairs() {
         match key.to_lowercase().as_bytes() {
-            b"cc" => tpl = tpl.cc(val),
-            b"bcc" => tpl = tpl.bcc(val),
-            b"subject" => tpl = tpl.subject(val),
-            b"body" => tpl = tpl.text_plain_part(val.as_bytes()),
+            b"cc" => builder = builder.cc(val.to_string()),
+            b"bcc" => builder = builder.bcc(val.to_string()),
+            b"subject" => builder = builder.subject(val),
+            b"body" => builder = builder.text_body(val),
             _ => (),
         }
     }
 
-    editor::edit_tpl_with_editor(config, printer, backend, sender, tpl.build())
+    let tpl = config
+        .generate_tpl_interpreter()
+        .show_only_headers(config.email_writing_headers())
+        .interpret_msg_builder(builder)?;
+
+    editor::edit_tpl_with_editor(config, printer, backend, sender, tpl)
 }
 
-pub fn move_<P: Printer, B: Backend + ?Sized>(
+pub fn move_<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     from_folder: &str,
     to_folder: &str,
     ids: Vec<&str>,
 ) -> Result<()> {
     let from_folder = config.folder_alias(from_folder)?;
     let to_folder = config.folder_alias(to_folder)?;
+    let ids = id_mapper.get_ids(ids)?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
     backend.move_emails(&from_folder, &to_folder, ids)?;
     printer.print("Email(s) successfully moved!")
 }
 
-pub fn read<P: Printer, B: Backend + ?Sized>(
+pub fn read<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     ids: Vec<&str>,
     text_mime: &str,
-    sanitize: bool,
     raw: bool,
     headers: Vec<&str>,
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
+    let ids = id_mapper.get_ids(ids)?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
     let emails = backend.get_emails(&folder, ids)?;
 
     let mut glue = "";
@@ -208,20 +236,15 @@ pub fn read<P: Printer, B: Backend + ?Sized>(
             // display what can be displayed
             bodies.push_str(&String::from_utf8_lossy(email.raw()?).into_owned());
         } else {
-            let tpl = email
-                .to_read_tpl_builder(config)?
-                .show_headers(config.email_reading_headers())
-                .show_headers(&headers)
-                .show_text_parts_only(true)
-                .use_show_text_parts_strategy(if text_mime == "plain" {
-                    ShowTextPartsStrategy::PlainOtherwiseHtml
-                } else {
-                    ShowTextPartsStrategy::HtmlOtherwisePlain
-                })
-                .sanitize_text_parts(sanitize)
-                .build();
-
-            bodies.push_str(&<Tpl as Into<String>>::into(tpl));
+            let tpl: String = email
+                .to_read_tpl(&config, |tpl| match text_mime {
+                    "html" => tpl
+                        .hide_all_headers()
+                        .filter_parts(FilterParts::Only("text/html".into())),
+                    _ => tpl.show_additional_headers(&headers),
+                })?
+                .into();
+            bodies.push_str(&tpl);
         }
 
         glue = "\n\n";
@@ -230,36 +253,43 @@ pub fn read<P: Printer, B: Backend + ?Sized>(
     printer.print(bodies)
 }
 
-pub fn reply<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn reply<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
     folder: &str,
     id: &str,
     all: bool,
-    headers: Option<Vec<&str>>,
+    headers: Option<Vec<(&str, &str)>>,
     body: Option<&str>,
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
+
+    let ids = id_mapper.get_ids([id])?;
+    let ids = ids.iter().map(String::as_str).collect::<Vec<_>>();
+
     let tpl = backend
-        .get_emails(&folder, vec![id])?
+        .get_emails(&folder, ids)?
         .first()
         .ok_or_else(|| anyhow!("cannot find email {}", id))?
-        .to_reply_tpl_builder(config, all)?
-        .set_some_raw_headers(headers)
-        .some_text_plain_part(body)
-        .build();
+        .to_reply_tpl_builder(config)
+        .some_headers(headers)
+        .some_body(body)
+        .reply_all(all)
+        .build()?;
     trace!("initial template: {}", *tpl);
     editor::edit_tpl_with_editor(config, printer, backend, sender, tpl)?;
     backend.add_flags(&folder, vec![id], &Flags::from_iter([Flag::Answered]))?;
     Ok(())
 }
 
-pub fn save<P: Printer, B: Backend + ?Sized>(
+pub fn save<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     raw_email: String,
 ) -> Result<()> {
@@ -276,14 +306,18 @@ pub fn save<P: Printer, B: Backend + ?Sized>(
             .collect::<Vec<String>>()
             .join("\r\n")
     };
-    backend.add_email(&folder, raw_email.as_bytes(), &Flags::default())?;
+
+    let id = backend.add_email(&folder, raw_email.as_bytes(), &Flags::default())?;
+    id_mapper.create_alias(id)?;
+
     Ok(())
 }
 
-pub fn search<P: Printer, B: Backend + ?Sized>(
+pub fn search<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     query: String,
     max_width: Option<usize>,
@@ -292,9 +326,10 @@ pub fn search<P: Printer, B: Backend + ?Sized>(
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
     let page_size = page_size.unwrap_or(config.email_listing_page_size());
-    let envelopes: Envelopes = backend
+    let mut envelopes: Envelopes = backend
         .search_envelopes(&folder, &query, "", page_size, page)?
         .into();
+    envelopes.remap_ids(id_mapper)?;
     let opts = PrintTableOpts {
         format: &config.email_reading_format,
         max_width,
@@ -303,10 +338,11 @@ pub fn search<P: Printer, B: Backend + ?Sized>(
     printer.print_table(Box::new(envelopes), opts)
 }
 
-pub fn sort<P: Printer, B: Backend + ?Sized>(
+pub fn sort<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
+    id_mapper: &IdMapper,
+    backend: &mut dyn Backend,
     folder: &str,
     sort: String,
     query: String,
@@ -316,9 +352,10 @@ pub fn sort<P: Printer, B: Backend + ?Sized>(
 ) -> Result<()> {
     let folder = config.folder_alias(folder)?;
     let page_size = page_size.unwrap_or(config.email_listing_page_size());
-    let envelopes: Envelopes = backend
+    let mut envelopes: Envelopes = backend
         .search_envelopes(&folder, &query, &sort, page_size, page)?
         .into();
+    envelopes.remap_ids(id_mapper)?;
     let opts = PrintTableOpts {
         format: &config.email_reading_format,
         max_width,
@@ -327,11 +364,11 @@ pub fn sort<P: Printer, B: Backend + ?Sized>(
     printer.print_table(Box::new(envelopes), opts)
 }
 
-pub fn send<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn send<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
     raw_email: String,
 ) -> Result<()> {
     let folder = config.sent_folder_alias()?;
@@ -349,26 +386,28 @@ pub fn send<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
     };
     trace!("raw email: {:?}", raw_email);
     sender.send(raw_email.as_bytes())?;
-    backend.add_email(
-        &folder,
-        raw_email.as_bytes(),
-        &Flags::from_iter([Flag::Seen]),
-    )?;
+    if config.email_sending_save_copy {
+        backend.add_email(
+            &folder,
+            raw_email.as_bytes(),
+            &Flags::from_iter([Flag::Seen]),
+        )?;
+    }
     Ok(())
 }
 
-pub fn write<P: Printer, B: Backend + ?Sized, S: Sender + ?Sized>(
+pub fn write<P: Printer>(
     config: &AccountConfig,
     printer: &mut P,
-    backend: &mut B,
-    sender: &mut S,
-    headers: Option<Vec<&str>>,
+    backend: &mut dyn Backend,
+    sender: &mut dyn Sender,
+    headers: Option<Vec<(&str, &str)>>,
     body: Option<&str>,
 ) -> Result<()> {
-    let tpl = Email::new_tpl_builder(config)?
-        .set_some_raw_headers(headers)
-        .some_text_plain_part(body)
-        .build();
+    let tpl = Email::new_tpl_builder(config)
+        .some_headers(headers)
+        .some_body(body)
+        .build()?;
     trace!("initial template: {}", *tpl);
     editor::edit_tpl_with_editor(config, printer, backend, sender, tpl)?;
     Ok(())
