@@ -1,6 +1,9 @@
 use anyhow::{anyhow, Context, Result};
 use clap::Command;
-use std::{borrow::Cow, env};
+use pimalaya_email::{
+    BackendBuilder, BackendConfig, ImapBackend, SenderBuilder, DEFAULT_INBOX_FOLDER,
+};
+use std::env;
 use url::Url;
 
 use himalaya::{
@@ -8,10 +11,7 @@ use himalaya::{
     config::{self, DeserializedConfig},
     email, flag, folder, man, output,
     printer::StdoutPrinter,
-    tpl,
-};
-use pimalaya_email::{
-    BackendBuilder, BackendConfig, ImapBackend, SenderBuilder, DEFAULT_INBOX_FOLDER,
+    tpl, IdMapper,
 };
 
 #[cfg(feature = "imap-backend")]
@@ -51,16 +51,16 @@ fn main() -> Result<()> {
     if raw_args.len() > 1 && raw_args[1].starts_with("mailto:") {
         let url = Url::parse(&raw_args[1])?;
         let config = DeserializedConfig::from_opt_path(None)?;
-        let (account_config, backend_config) = config.to_configs(None)?;
-        let mut backend = BackendBuilder::new().build(&account_config, &backend_config)?;
-        let mut sender = SenderBuilder::build(&account_config)?;
+        let account_config = config.to_account_config(None)?;
+        let mut backend = BackendBuilder::new().build(&account_config)?;
+        let mut sender = SenderBuilder::new().build(&account_config)?;
         let mut printer = StdoutPrinter::default();
 
         return email::handlers::mailto(
             &account_config,
-            &mut printer,
             backend.as_mut(),
             sender.as_mut(),
+            &mut printer,
             &url,
         );
     }
@@ -87,25 +87,23 @@ fn main() -> Result<()> {
 
     // inits config
     let config = DeserializedConfig::from_opt_path(config::args::parse_arg(&m))?;
-    let (account_config, backend_config) = config.to_configs(account::args::parse_arg(&m))?;
+    let account_config = config.to_account_config(account::args::parse_arg(&m))?;
     let folder = folder::args::parse_source_arg(&m);
 
     // checks IMAP commands
     #[cfg(feature = "imap-backend")]
-    if let BackendConfig::Imap(imap_config) = &backend_config {
+    if let BackendConfig::Imap(imap_config) = &account_config.backend {
         let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
 
         // FIXME: find a way to downcast `backend` instead of
         // recreating an instance.
         match imap::args::matches(&m)? {
             Some(imap::args::Cmd::Notify(keepalive)) => {
-                let imap =
-                    ImapBackend::new(Cow::Borrowed(&account_config), Cow::Borrowed(&imap_config))?;
+                let imap = ImapBackend::new(account_config.clone(), imap_config.clone())?;
                 return imap::handlers::notify(&imap, &folder, keepalive);
             }
             Some(imap::args::Cmd::Watch(keepalive)) => {
-                let imap =
-                    ImapBackend::new(Cow::Borrowed(&account_config), Cow::Borrowed(&imap_config))?;
+                let imap = ImapBackend::new(account_config.clone(), imap_config.clone())?;
                 return imap::handlers::watch(&imap, &folder, keepalive);
             }
             _ => (),
@@ -113,9 +111,8 @@ fn main() -> Result<()> {
     }
 
     // inits services
-    let mut sender = SenderBuilder::build(&account_config)?;
-    let mut printer = StdoutPrinter::try_from(&m)?;
     let disable_cache = cache::args::parse_disable_cache_flag(&m);
+    let mut printer = StdoutPrinter::try_from(&m)?;
 
     // checks account commands
     match account::args::matches(&m)? {
@@ -126,7 +123,7 @@ fn main() -> Result<()> {
             let backend = BackendBuilder::new()
                 .sessions_pool_size(8)
                 .disable_cache(true)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
             account::handlers::sync(
                 &account_config,
                 &mut printer,
@@ -137,6 +134,9 @@ fn main() -> Result<()> {
             backend.close()?;
             return Ok(());
         }
+        Some(account::args::Cmd::Configure(reset)) => {
+            return account::handlers::configure(&account_config, reset);
+        }
         _ => (),
     }
 
@@ -149,13 +149,13 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder)?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
             return folder::handlers::create(&mut printer, backend.as_mut(), &folder);
         }
         Some(folder::args::Cmd::List(max_width)) => {
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
             return folder::handlers::list(
                 &account_config,
                 &mut printer,
@@ -167,7 +167,7 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
             return folder::handlers::expunge(&mut printer, backend.as_mut(), &folder);
         }
         Some(folder::args::Cmd::Delete) => {
@@ -177,7 +177,7 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder)?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
             return folder::handlers::delete(&mut printer, backend.as_mut(), &folder);
         }
         _ => (),
@@ -189,10 +189,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::attachments(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 ids,
@@ -202,10 +204,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::copy(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 to_folder,
@@ -216,10 +220,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::delete(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 ids,
@@ -229,10 +235,13 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let mut sender = SenderBuilder::new().build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::forward(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 sender.as_mut(),
                 &folder,
@@ -245,10 +254,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::list(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 max_width,
@@ -260,29 +271,32 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::move_(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 to_folder,
                 ids,
             );
         }
-        Some(email::args::Cmd::Read(ids, text_mime, sanitize, raw, headers)) => {
+        Some(email::args::Cmd::Read(ids, text_mime, raw, headers)) => {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::read(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 ids,
                 text_mime,
-                sanitize,
                 raw,
                 headers,
             );
@@ -291,10 +305,13 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let mut sender = SenderBuilder::new().build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::reply(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 sender.as_mut(),
                 &folder,
@@ -308,10 +325,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::save(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 raw_email,
@@ -321,10 +340,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::search(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 query,
@@ -337,10 +358,12 @@ fn main() -> Result<()> {
             let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
             return email::handlers::sort(
                 &account_config,
                 &mut printer,
+                &id_mapper,
                 backend.as_mut(),
                 &folder,
                 criteria,
@@ -353,7 +376,8 @@ fn main() -> Result<()> {
         Some(email::args::Cmd::Send(raw_email)) => {
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let mut sender = SenderBuilder::new().build(&account_config)?;
             return email::handlers::send(
                 &account_config,
                 &mut printer,
@@ -367,22 +391,46 @@ fn main() -> Result<()> {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
-                return flag::handlers::set(&mut printer, backend.as_mut(), &folder, ids, flags);
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
+                return flag::handlers::set(
+                    &mut printer,
+                    &id_mapper,
+                    backend.as_mut(),
+                    &folder,
+                    ids,
+                    flags,
+                );
             }
             Some(flag::args::Cmd::Add(ids, ref flags)) => {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
-                return flag::handlers::add(&mut printer, backend.as_mut(), &folder, ids, flags);
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
+                return flag::handlers::add(
+                    &mut printer,
+                    &id_mapper,
+                    backend.as_mut(),
+                    &folder,
+                    ids,
+                    flags,
+                );
             }
             Some(flag::args::Cmd::Remove(ids, ref flags)) => {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
-                return flag::handlers::remove(&mut printer, backend.as_mut(), &folder, ids, flags);
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
+                return flag::handlers::remove(
+                    &mut printer,
+                    &id_mapper,
+                    backend.as_mut(),
+                    &folder,
+                    ids,
+                    flags,
+                );
             }
             _ => (),
         },
@@ -391,10 +439,12 @@ fn main() -> Result<()> {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
                 return tpl::handlers::forward(
                     &account_config,
                     &mut printer,
+                    &id_mapper,
                     backend.as_mut(),
                     &folder,
                     id,
@@ -409,10 +459,12 @@ fn main() -> Result<()> {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
                 return tpl::handlers::reply(
                     &account_config,
                     &mut printer,
+                    &id_mapper,
                     backend.as_mut(),
                     &folder,
                     id,
@@ -425,10 +477,12 @@ fn main() -> Result<()> {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
+                    .build(&account_config)?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config.name, &folder)?;
                 return tpl::handlers::save(
                     &account_config,
                     &mut printer,
+                    &id_mapper,
                     backend.as_mut(),
                     &folder,
                     tpl,
@@ -438,7 +492,8 @@ fn main() -> Result<()> {
                 let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
                 let mut backend = BackendBuilder::new()
                     .disable_cache(disable_cache)
-                    .build(&account_config, &backend_config)?;
+                    .build(&account_config)?;
+                let mut sender = SenderBuilder::new().build(&account_config)?;
                 return tpl::handlers::send(
                     &account_config,
                     &mut printer,
@@ -453,7 +508,8 @@ fn main() -> Result<()> {
         Some(email::args::Cmd::Write(headers, body)) => {
             let mut backend = BackendBuilder::new()
                 .disable_cache(disable_cache)
-                .build(&account_config, &backend_config)?;
+                .build(&account_config)?;
+            let mut sender = SenderBuilder::new().build(&account_config)?;
             return email::handlers::write(
                 &account_config,
                 &mut printer,
